local mod = ...



-- Building Asset Processor
mod:registerAssetProcessor("models/Influence1.fbx", { DataType = "BUILDING_ASSET_PROCESSOR" })



-- Prefab -----------------------------------------------------------------------------------------------------------
mod:registerAssetId("models/Influence1.fbx/Prefab/Handcart.Part", "PREFAB_IAE_HANDCART")
mod:registerAssetId("models/Influence1.fbx/Prefab/Handcart.Part/Generator.Food", "PREFAB_IAE_HANDCART_GENFOOD")
mod:registerAssetId("models/Influence1.fbx/Prefab/Handcart.Part/Generator.Water", "PREFAB_IAE_HANDCART_GENWATER")
mod:registerAssetId("models/Influence1.fbx/Prefab/Handcart.Part/Generator.Wood", "PREFAB_IAE_HANDCART_GENCOAL")



-- Building -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "BUILDING",
	Id = "BUILDING_IAE_HANDCART",
	Name = "BUILDING_IAE_HANDCART_NAME",
	Description = "BUILDING_IAE_HANDCART_DESC",
	BuildingType = "LOGISTICS",
    AssetCoreBuildingPart = "BUILDING_PART_IAE_HANDCART",
	IsEditable = false,
    IsManuallyUnlocked = true,
    IsUnique = true,
})



-- Building Part -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_IAE_HANDCART",
	Name = "BUILDING_PART_IAE_HANDCART_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_IAE_HANDCART"
	},
	BuildingZone = {
        ZoneEntryList = {
            { 
                Polygon = polygon.createRectangle( {1.5, 2.0}, {0, 0} ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = false }
			},
		}
	},
    AssetBuildingFunction = "BUILDING_FUNCTION_BUILDER_WORKSHOP",
})



-- Building Function: Bread  -------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_RESOURCE_GENERATOR",
    Id = "BUILDING_FUNCTION_IAE_GENFOOD",
	Name = "BUILDING_FUNCTION_IAE_GENFOOD_NAME",
	ResourceGenerated = "BREAD",
	IsForConsumer = true,
	IsInfinite = false,
})

local COMP_IAE_HANDCART_GENFOOD = {
	TypeName = "COMP_IAE_HANDCART_GENFOOD",
	ParentType = "COMPONENT",
	Properties = {
    }
}

function COMP_IAE_HANDCART_GENFOOD:init()

    -- Set the resource generator
    local compPart = self:getOwner():getOrCreateComponent("COMP_BUILDING_PART")
    local compResGen = self:getOwner():getOrCreateComponent("COMP_RESOURCE_GENERATOR")
    compResGen:setResourceGeneratorData("BUILDING_FUNCTION_IAE_GENFOOD")

end

mod:registerClass(COMP_IAE_HANDCART_GENFOOD)

mod:registerPrefabComponent("PREFAB_IAE_HANDCART_GENFOOD", { DataType = "COMP_BUILDING_PART"})
mod:registerPrefabComponent("PREFAB_IAE_HANDCART_GENFOOD", { DataType = "COMP_RESOURCE_GENERATOR", GrowRate = 0, MaxQuantity = 40 })
mod:registerPrefabComponent("PREFAB_IAE_HANDCART_GENFOOD", { DataType = "COMP_IAE_HANDCART_GENFOOD"})



-- Building Function: Water  -------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_RESOURCE_GENERATOR",
    Id = "BUILDING_FUNCTION_IAE_GENWATER",
	Name = "BUILDING_FUNCTION_IAE_GENWATER_NAME",
	ResourceGenerated = "WATER",
	IsForConsumer = true,
	IsInfinite = false,
})

local COMP_IAE_HANDCART_GENWATER = {
	TypeName = "COMP_IAE_HANDCART_GENWATER",
	ParentType = "COMPONENT",
	Properties = {
    }
}

function COMP_IAE_HANDCART_GENWATER:init()

    -- Set the resource generator
    local compPart = self:getOwner():getOrCreateComponent("COMP_BUILDING_PART")
    local compResGen = self:getOwner():getOrCreateComponent("COMP_RESOURCE_GENERATOR")
    compResGen:setResourceGeneratorData("BUILDING_FUNCTION_IAE_GENWATER")

end

mod:registerClass(COMP_IAE_HANDCART_GENWATER)

mod:registerPrefabComponent("PREFAB_IAE_HANDCART_GENWATER", { DataType = "COMP_BUILDING_PART"})
mod:registerPrefabComponent("PREFAB_IAE_HANDCART_GENWATER", { DataType = "COMP_RESOURCE_GENERATOR", GrowRate = 0, MaxQuantity = 40 })
mod:registerPrefabComponent("PREFAB_IAE_HANDCART_GENWATER", { DataType = "COMP_IAE_HANDCART_GENWATER"})



-- Building Function: Coal (for fuel if needed) -------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_RESOURCE_GENERATOR",
    Id = "BUILDING_FUNCTION_IAE_GENCOAL",
	Name = "BUILDING_FUNCTION_IAE_GENCOAL_NAME",
	ResourceGenerated = "COAL",
	IsForConsumer = true,
	IsInfinite = false,
})

local COMP_IAE_HANDCART_GENCOAL = {
	TypeName = "COMP_IAE_HANDCART_GENCOAL",
	ParentType = "COMPONENT",
	Properties = {
    }
}

function COMP_IAE_HANDCART_GENCOAL:init()

    -- Set the resource generator
    local compPart = self:getOwner():getOrCreateComponent("COMP_BUILDING_PART")
    local compResGen = self:getOwner():getOrCreateComponent("COMP_RESOURCE_GENERATOR")
    compResGen:setResourceGeneratorData("BUILDING_FUNCTION_IAE_GENCOAL")

end

mod:registerClass(COMP_IAE_HANDCART_GENCOAL)

mod:registerPrefabComponent("PREFAB_IAE_HANDCART_GENCOAL", { DataType = "COMP_BUILDING_PART"})
mod:registerPrefabComponent("PREFAB_IAE_HANDCART_GENCOAL", { DataType = "COMP_RESOURCE_GENERATOR", GrowRate = 0, MaxQuantity = 40 })
mod:registerPrefabComponent("PREFAB_IAE_HANDCART_GENCOAL", { DataType = "COMP_IAE_HANDCART_GENCOAL"})



-- Component: Hide the handcart after it is built -------------------------------------------------------------------------------------
local COMP_IAE_HANDCART = {
	TypeName = "COMP_IAE_HANDCART",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "InitFlag", Type = "boolean", Default = false, Flags = { "SAVE_GAME" } },
    }
}


function COMP_IAE_HANDCART:init()
    -- mod:log("Init statement fired for handcart")
    self:lockHandcartBuilding()

    -- set up daily event (since it sometimes reappears, not sure why...)
    local compMainGameLoop = self:getLevel():find("COMP_MAIN_GAME_LOOP")
    event.register(self, compMainGameLoop.ON_NEW_DAY, function()

        self:lockHandcartBuilding()

    end)

end


function COMP_IAE_HANDCART:lockHandcartBuilding()
    -- Lock the handcart building
    local compBM = self:getLevel():getComponentManager("COMP_BUILDING_MANAGER"):getFirst()
    local compBuilding = self:getOwner():findFirstObjectWithComponentUp("COMP_BUILDING")
    if (compBuilding) and (compBM) then
        local assetBuilding = compBuilding.AssetBuilding
        if (assetBuilding) then
            compBM:lockBuilding(assetBuilding)
        end
    end
end

mod:registerClass(COMP_IAE_HANDCART)

mod:registerPrefabComponent("PREFAB_IAE_HANDCART", { DataType = "COMP_IAE_HANDCART" } )




